#!/usr/bin/perl -w	

##############################################################################
#
# Print billing management system - web tools, version 4.2.0
#
# Copyright (C) 2000, 2001, 2002, 2003 Daniel Franklin
#
# This program is distributed under the terms of the GNU General Public
# License Version 2.
#
# This CGI script allows users to check their quota via the web.
#
##############################################################################

use Locale::gettext;
use POSIX;
use Printbill::printbill_pcfg;
use strict;

setlocale (LC_MESSAGES, "");
textdomain ("printbill");

my $query = $ENV{'QUERY_STRING'};
my $request_method = $ENV{'REQUEST_METHOD'};
my (%queries, $config, %params, $quota, %formdata);
&parse_query_data (\%queries, $query);

$config = "/etc/printbill/printbillrc";
%params = pcfg ($config);

print "Content-type: text/html\n\n";

print `/bin/cat $params{'web_header'}`;

if ($? != 0) {
	&return_error (500, "CGI Error",
		sprintf (gettext ("%s: Error: cannot read web header file %s.\n"), $0, $params{'web_header'}));
}

if ($request_method eq "POST") {
	&parse_form_data (\%formdata);
	if ( ! $formdata{'username'}) {
		&setup_user_check ();
	} else {
		&setup_user_page ($formdata{'username'});
	}
} else {
	&setup_user_check ();
}

print `/bin/cat $params{'web_footer'}`;

if ($? != 0) {
	&return_error (500, "CGI Error",
		sprintf (gettext ("%s: Error: cannot read web footer file %s.\n"), $0, $params{'web_footer'}));
}

exit 0;

sub parse_query_data
{
	my ($query_data, $command) = @_;
	my (@key_value_pairs, $key_value, $key, $value);

	@key_value_pairs = split (/&/, $command);
	
	foreach $key_value (@key_value_pairs) {
		($key, $value) = split (/=/, $key_value);
		$value =~ tr/+/ /;
		$value =~ s/%([\dA-Fa-f][\dA-Fa-f])/pack ("C", hex ($1))/eg;
		
		$$query_data{$key} = $value;
	}
}

sub parse_form_data
{
	my ($form_data) = @_;
	my ($post_info, @key_value_pairs, $key_value, $key, $value);
	
	read (STDIN, $post_info, $ENV{'CONTENT_LENGTH'});

	@key_value_pairs = split (/&/, $post_info);
	
	foreach $key_value (@key_value_pairs) {
		($key, $value) = split (/=/, $key_value);
		$value =~ tr/+/ /;
		$value =~ s/%([\dA-Fa-f][\dA-Fa-f])/pack ("C", hex ($1))/eg;
		$value =~ s/;//g;

		$$form_data{$key} = $value;
	}
}

sub return_error
{
	my ($status, $keyword, $message) = @_;

	printf "<title>%s</title>", gettext ("CGI Program - Unexpected Error");
	print "<h1>$keyword</h1><hr>$message<hr>";
	printf "Status: $status $keyword\n\n", gettext ("Status");
	print "</body></html>";

	exit 1;
}

sub setup_user_check
{
	# setup header information
	print `/bin/cat $params{'web_message_text'}`
		or &return_error (500, "CGI Error",
			sprintf (gettext ("%s: Error: cannot read web message text file %s.\n"), $0, $params{'web_message_text'}));
	
	printf "<center><h2>%s</h2></center>\n", gettext ("Print Quota System");

	print "<hr>\n";
	print "<form action=\"$params{'check_cgi'}\" method=\"POST\">\n";
	printf "<br>%s<br>\n", gettext ("Please enter your username:");
	print "<center><table border=0>\n";
	printf "<tr><td width=100 align=left>%s:</td><td width=100 align=right><input type=\"text\" name=\"username\" SIZE=\"10\"></td></tr>\n", gettext ("Username");
	printf "<tr><td align=left><input type=\"submit\" value=\"%s\"></td>\n", gettext ("Submit");
	printf "<td align=right><input type=\"reset\" value=\"%s\">\n", gettext ("Reset");
	print "<input type=\"hidden\" name=\"ftype\" value=\"usercheck\"></td></tr>\n";
	print "</table></center>\n";
	print "<hr></form>\n";
}

sub setup_user_page
{
	my ($user) = @_;
	my ($pages, $spent, $quota, $answer);

	print `/bin/cat $params{'web_message_text'}`
		or &return_error (500, "CGI Error",
			sprintf (gettext ("%s: Error: cannot read web message text file %s.\n"), $0, $params{'web_message_text'}));
	
	printf "<center><h2>%s</h2></center>\n", gettext ("Print Quota System");
	print "<hr>\n";

	$user =~ s/\'/\\\'/g;

	print "<p><center><table cellpadding=2 cellspacing=2 border=1>\n";
	printf "<thead><td><b>%s</b></td>\n", gettext ("Username");
	printf "<td><b>%s</b></td>\n", gettext ("Quota Remaining");
	printf "<td><b>%s</b></td>\n", gettext ("Total Spent");
	printf "<td><b>%s</b></td>\n", gettext ("Pages Printed");
	print "</thead>\n";
	print "<tbody>";

	$answer = `$params{'admin_prog_path'}/pqm --display \'$user\' --web 2>&1`;

	if ($? != 0) {
		&return_error (500, "CGI Error",
			sprintf (gettext ("%s: Error: cannot execute %s: %s.\n"), $0, "$params{'admin_prog_path'}/pqm --display \'$user\' --web", $answer));
	} else {
		$quota = $answer;
	}
	
	$answer = `$params{'admin_prog_path'}/pqm --pages \'$user\' --web 2>&1`;

	if ($? != 0) {
		&return_error (500, "CGI Error",
			sprintf (gettext ("%s: Error: cannot execute %s: %s.\n"), $0, "$params{'admin_prog_path'}/pqm --pages \'$user\' --web", $answer));
	} else {
		$pages = $answer;
	}

	$answer = `$params{'admin_prog_path'}/pqm --spent \'$user\' --web 2>&1`;

	if ($? != 0) {
		&return_error (500, "CGI Error",
			sprintf (gettext ("%s: Error: cannot execute %s: %s.\n"), $0, "$params{'admin_prog_path'}/pqm --spent \'$user\' --web", $answer));
	} else {
		$spent = $answer;
	}

# Attempt to make the resulting HTML a bit more readable...
		
	chop $quota;
	chop $pages;
	chop $spent;
		
	print "<tr><td>$user</td>\n<td>$quota</td>\n";
	print "<td>$spent</td>\n<td>$pages</td>\n</tr>";
	print "</tbody></table></center>\n";

	print "<center>\n";
	print "<form action=\"$params{'check_cgi'}\" method=\"POST\">\n";
	printf "<input type=\"submit\" value=\"%s\">\n", gettext ("Reload");
	print "<input type=\"hidden\" name=\"ftype\" value=\"usercheck\">\n";
	print "<input type=\"hidden\" name=\"username\" value=\"$user\">\n";
	print "</form></center>\n";

	print `/bin/cat $params{'extra_quota_message'}`;
	
	if ($? != 0) {
		&return_error (500, "CGI Error",
			sprintf (gettext ("%s: Error: cannot read extra quota message file %s.\n"), $0, $params{'extra_quota_text'}));
	}
}
